/*****************************************************************************
 *                                                                           *
 *   Copyright (C) 2005 by Chazal Francois             <neptune3k@free.fr>   *
 *                 2007 by Andrey Skvortsov            <stingo000@yahoo.ca>  *
 *                 based on Quadconsole 2.0.1 code by                        *
 *                 Simon Perreault                                           *
 *                                                                           *
 *                                                                           *
 *                     =========  GPL Licence  =========                     *
 *    This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the  GNU General Public License as published by   *
 *   the  Free  Software  Foundation ; either version 2 of the License, or   *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *****************************************************************************/

//== INCLUDE REQUIREMENTS =====================================================


#include <qapplication.h>
#include <qlayout.h>

#include <kdebug.h>
/*
** Local libraries */
#include <kparts/part.h>
#include <klibloader.h>

#include <algorithm>

#include "shell_session.h"
#include "shell_session.moc"


//== CONSTRUCTORS AND DESTRUCTORS =============================================

ShellSession::ShellSession(QWidget * parent, int nlay , const char * name) : QObject(parent, name)
{
      int ncols, nrows;                    
      session_widget   = NULL;            
      part_selected_id = 0;
            
      if ((mFactory = KLibLoader::self()->factory("libkonsolepart")) == NULL)          
          kdFatal() << "No libkonsolepart found !" << endl;
         
      session_widget= new QWidget(parent);
   
      switch(nlay)
      {
        case 4: nrows = 2;  ncols =2; break;
        case 2: nrows = 1;  ncols =2; break;
        case 3: nrows = 2;  ncols =1; break;
        default: nrows = 1; ncols =1; break;
      }
      
      mLayout = new QGridLayout(session_widget, nrows, ncols, 0, 1 );                  
      //mFilter = new MouseMoveFilter(this);
                 
      for ( int i = 0; i < nrows; ++i ) 
      for ( int j = 0; j < ncols; ++j ) {
            mKonsoleParts.push_back( createPart(i,j, session_widget) );
      }
                  
      mgoRight = new QAction( QString::null, CTRL + SHIFT + Key_Right,this );
      connect( mgoRight, SIGNAL(activated()), this, SLOT(focusKonsoleRight()) );
      
      mgoLeft = new QAction( QString::null, CTRL + SHIFT + Key_Left, this);
      connect( mgoLeft, SIGNAL(activated()), this, SLOT(focusKonsoleLeft()) );
      
      mgoUp = new QAction( QString::null, CTRL + SHIFT + Key_Up, this );
      connect( mgoUp, SIGNAL(activated()), this, SLOT(focusKonsoleUp()) );
      
      mgoDown = new QAction( QString::null, CTRL + SHIFT + Key_Down, this);
      connect( mgoDown, SIGNAL(activated()), this, SLOT(focusKonsoleDown()) );
    
}


KParts::Part* ShellSession::createPart( int row, int column, QWidget* wsession)
{

    KParts::Part* part =(KParts::ReadOnlyPart *) mFactory->create(wsession);
    connect( part, SIGNAL(destroyed()), SLOT(partDestroyed()) );

    // Make this part's widget activate on mouse over
  //  if (mFilter) {
       // part->widget()->setMouseTracking(true);
    //    part->widget()->installEventFilter(mFilter);
        //part->widget()->setFocusPolicy(QWidget::WheelFocus);
   // }

    mLayout->addWidget( part->widget(), row, column );
    part->widget()->show(); 
   // kdWarning() << "created!" << endl;  
    return part;
}


class PartDeletedEvent : public QCustomEvent
{
    public:
        PartDeletedEvent( int row, int column )
            : QCustomEvent( QEvent::User + 1 )
            , mRow(row)
            , mColumn(column)
        {
        }

        inline int row() const { return mRow; }
        inline int column() const { return mColumn; }

    private:
        int mRow, mColumn;
};


void ShellSession::partDestroyed()
{
   // kdDebug() << k_funcinfo << endl;
    PartVector::iterator part = std::find( mKonsoleParts.begin(),
            mKonsoleParts.end(), sender() );
    if ( part != mKonsoleParts.end() ) {
        *part = 0;

        // A terminal emulator has been destroyed. Plan to start a new one.
        int index = part - mKonsoleParts.begin();
        int row = index / mLayout->numCols();
        int col = index % mLayout->numCols();

        QApplication::postEvent( this, new PartDeletedEvent(row, col) );
    }
    //kdWarning() << "deleted!" << endl;
}

void ShellSession::customEvent( QCustomEvent* e )
{
   // kdDebug() << k_funcinfo << endl;
    if ( e->type() == QEvent::User + 1 ) {
        // Start a new part.
        PartDeletedEvent* pde = static_cast<PartDeletedEvent*>(e);
        mKonsoleParts[ pde->row() * mLayout->numCols() + pde->column() ] =
            createPart( pde->row(), pde->column(), session_widget ); }      
}


ShellSession::~ShellSession()
{  
    for ( PartVector::const_iterator i = mKonsoleParts.begin(); i !=
            mKonsoleParts.end(); ++i ) {                 
        delete *i;        
     //kdWarning() << "destroying shell!" << endl;
    }    
    delete session_widget;

    delete mgoRight;
    delete mgoLeft;
    delete mgoUp;
    delete mgoDown;
  
}


void ShellSession::focusKonsoleRight()
{
    for ( PartVector::iterator part = mKonsoleParts.begin(); part !=
            mKonsoleParts.end(); ++part ) {
        if ( (*part)->widget()->hasFocus() ) {
            ++part;
            if ( part == mKonsoleParts.end() ) {
                part = mKonsoleParts.begin();
            }
            (*part)->widget()->setFocus();
            break;
        }
    }
}

void ShellSession::focusKonsoleLeft()
{   
    for ( PartVector::reverse_iterator part = mKonsoleParts.rbegin(); part !=
            mKonsoleParts.rend(); ++part ) {
        if ( (*part)->widget()->hasFocus() ) {
            ++part;
            if ( part == mKonsoleParts.rend() ) {
                part = mKonsoleParts.rbegin();
            }
            (*part)->widget()->setFocus();
            break;
        }
    }
}

void ShellSession::focusKonsoleUp()
{
  int nmaxR = mLayout->numRows();
  int nmaxC = mLayout->numCols();
  for ( int i = 0; i < nmaxR; ++i ) {
  for ( int j = 0; j < nmaxC; ++j ) {
        if ( mKonsoleParts[i*nmaxC+j]->widget()->hasFocus() ) {
                if ( i == 0 )
                    i = nmaxR - 1;                
                else 
                   --i;               
                mKonsoleParts[i*nmaxC+j]->widget()->setFocus();
                break;
         }
  }}

}

void ShellSession::focusKonsoleDown()
{
  int nmaxR = mLayout->numRows();
  int nmaxC = mLayout->numCols();
  for ( int i = 0; i < nmaxR; ++i ){
  for ( int j = 0; j < nmaxC; ++j ){
        if ( mKonsoleParts[i*nmaxC+j]->widget()->hasFocus()){
                if (i == nmaxR - 1)
                  i = 0;                
                else 
                  ++i;                
                mKonsoleParts[i*nmaxC+j]->widget()->setFocus();
                break;
         }
   }}
            
}

//===============================================================

void ShellSession::KeepPartsSlectedId()
{
  int nmaxR = mLayout->numRows();
  int nmaxC = mLayout->numCols();
  for ( int i = 0; i < nmaxR; ++i )
  for ( int j = 0; j < nmaxC; ++j )          
          if ( mKonsoleParts[ nmaxC*i + j]->widget()->hasFocus() )
                 part_selected_id = nmaxC*i+ j;
                  
}

void ShellSession::FocusSession()
{  
  mKonsoleParts[part_selected_id]->widget()->setFocus();
}

int ShellSession::PartSelected()
{ 
   return part_selected_id;
}
