/*****************************************************************************
 *                                                                           *
 *   Copyright (C) 2005 by Chazal Francois             <neptune3k@free.fr>   *
 *                 2007 by Andrey Skvortsov            <stingo000@yahoo.ca>  *
 *                                                                           *
 *                     =========  GPL Licence  =========                     *
 *    This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the  GNU General Public License as published by   *
 *   the  Free  Software  Foundation ; either version 2 of the License, or   *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *****************************************************************************/

//== INCLUDE REQUIREMENTS =====================================================

/*
** Local libraries */
#include "title_bar.h"
#include "title_bar.moc"



//== CONSTRUCTORS AND DESTRUCTORS =============================================


TitleBar::TitleBar(QWidget * parent, const char * name, const QString & skin) : QWidget(parent, name)
{
    loadSkin(skin);
}

TitleBar::~TitleBar()
{

}

//== PUBLIC METHODS ===========================================================


/******************************************************************************
** Gets the title bar's mask
******************************/

QRegion &   TitleBar::getWidgetMask()
{
    return mask;
}


/******************************************************************************
** Sets the title bar's text
******************************/

void        TitleBar::setTitleText(const QString & text)
{
    text_value = text + text_title;
    repaint();
}


/******************************************************************************
** Sets the configuration menu
********************************/

// void    TitleBar::setConfigurationMenu(KPopupMenu * menu)
// {
//    
// }



//== PROTECTED METHODS ========================================================


/******************************************************************************
** Repaints the widget when asked
***********************************/

void    TitleBar::paintEvent(QPaintEvent *)
{
    QPainter    painter(this);
    QFont       font(painter.font());

    painter.drawTiledPixmap(0, 0, width(), height(), back_image);

    painter.drawPixmap(0, 0, left_corner);
    painter.drawPixmap(width() - right_corner.width(), 0, right_corner);

    font.setBold(true);
    painter.setFont(font);
    painter.setPen(text_color);
    painter.drawText(text_position, text_value);

    painter.end();
}


/******************************************************************************
** Adjusts the widget's components when resized
*************************************************/

void    TitleBar::resizeEvent(QResizeEvent *)
{
    updateWidgetMask();
}

//== PRIVATE METHODS ==========================================================


/******************************************************************************
** Initializes the skin objects
*********************************/

void    TitleBar::loadSkin(const QString & skin)
{
    QUrl    url(locate("appdata", skin + "/title.skin"));
    KConfig config(url.path());

    // Loads the text informations ------------------------

    config.setGroup("Text");

    text_title = text_value = config.readEntry("text", 0);

    text_color.setRgb(config.readNumEntry("red", 0),
                      config.readNumEntry("green", 0),
                      config.readNumEntry("blue", 0));

    text_position.setX(config.readNumEntry("x", 0));
    text_position.setY(config.readNumEntry("y", 0));

    // Loads the background pixmaps -----------------------

    config.setGroup("Background");

    back_image.load(url.dirPath() + config.readEntry("back_image", ""));
    left_corner.load(url.dirPath() + config.readEntry("left_corner", ""));
    right_corner.load(url.dirPath() + config.readEntry("right_corner", ""));

    // Resizes the widgets --------------------------------

    resize(width(), back_image.height());
}


/******************************************************************************
** Updates the widget's mask
******************************/

void    TitleBar::updateWidgetMask()
{
    QRegion     temp_mask;

    mask = QRegion(*left_corner.mask());
    mask += QRegion(QRect(QPoint(left_corner.width(), 0), QPoint(width() - right_corner.width() - 1, height() - 1)));

    temp_mask = QRegion(*right_corner.mask());
    temp_mask.translate(width() - right_corner.width(), 0);
    mask += temp_mask;
}
